function dX = TorqueChangeSystemDiff(t,X)

% State-costate dynamic equation. The whole system state is:
% X=[theta1 theta2 thetadot1 thetadot2 tau1 tau2 p1 p2 p3 p4 p5 p6]

% The first set of equations (state dynamics) consits in the system dynamics:
%   M*d2q + C*dq + B*dq + g = tau
%   dtau = u
% driven by the optimal control input: 
%   u = -[p5; p6]

% The second set of equations consists in the costate dynamics:
%   dp = f(x)
% where the function f(.) is computed in symbolicComputations.m

global m1 m2 c2 c1 l1 Icm1 Icm2 g B11 B12 B21 B22

%rearranging the state
q1   = X(1);
q2   = X(2);
dq1  = X(3);
dq2  = X(4);
tau1 = X(5);
tau2 = X(6);

%rearranging the costate
p1   = X(7);
p2   = X(8);
p3   = X(9);
p4   = X(10);
p5   = X(11);
p6   = X(12);

%% here we simply copy-paste the results from symbolicComputations.m
alpha = m1*c1^2 + Icm1 + Icm2 + m2*(c2^2 + l1^2);
beta  = c2*l1*m2;
delta = m2*c2^2 + Icm2;

Minv11 = -delta/(beta^2*cos(q2)^2 + delta^2 - alpha*delta);
Minv12 = (delta + beta*cos(q2))/(beta^2*cos(q2)^2 + delta^2 - alpha*delta);
Minv22 = -(alpha + 2*beta*cos(q2))/(beta^2*cos(q2)^2 + delta^2 - alpha*delta);

h = c2*l1*m2*sin(q2);
C11 = -dq2*h;
C12 = -h*(dq1 + dq2);
C21 = dq1*h;
C22 = 0;

g1 = g*m2*(c2*cos(q1 + q2) + l1*cos(q1)) + c1*g*m1*cos(q1);
g2 = c2*g*m2*cos(q1 + q2);

dg1_dq1 = - g*m2*(c2*sin(q1 + q2) + l1*sin(q1)) - c1*g*m1*sin(q1);
dg1_dq2 = -c2*g*m2*sin(q1 + q2);
dg2_dq1 = -c2*g*m2*sin(q1 + q2);
dg2_dq2 = -c2*g*m2*sin(q1 + q2);

dh_dq2 = c2*l1*m2*cos(q2);

dMinv11_dq2 = -(2*beta^2*delta*cos(q2)*sin(q2))/(beta^2*cos(q2)^2 + delta^2 - alpha*delta)^2;
dMinv12_dq2 = (2*beta^2*cos(q2)*sin(q2)*(delta + beta*cos(q2)))/(beta^2*cos(q2)^2 + delta^2 - alpha*delta)^2 - (beta*sin(q2))/(beta^2*cos(q2)^2 + delta^2 - alpha*delta);
dMinv22_dq2 = (2*beta*sin(q2))/(beta^2*cos(q2)^2 + delta^2 - alpha*delta) - (2*beta^2*cos(q2)*sin(q2)*(alpha + 2*beta*cos(q2)))/(beta^2*cos(q2)^2 + delta^2 - alpha*delta)^2;

u1 = -p5;
u2 = -p6;

dx1 = dq1;
dx2 = dq2;
dx3 = Minv11*(tau1 - B11*dq1 - B12*dq2 - g1 + dq2^2*h + 2*dq1*dq2*h) - Minv12*(h*dq1^2 + B21*dq1 - tau2 + B22*dq2 + g2);
dx4 = Minv12*(tau1 - B11*dq1 - B12*dq2 - g1 + dq2^2*h + 2*dq1*dq2*h) - Minv22*(h*dq1^2 + B21*dq1 - tau2 + B22*dq2 + g2);
dx5 = u1;
dx6 = u2;

dp1 = (p3*Minv11 + p4*Minv12)*dg1_dq1 + (p3*Minv12 + p4*Minv22)*dg2_dq1;
dp2 = (p3*dMinv12_dq2 + p4*dMinv22_dq2)*(h*dq1^2 + B21*dq1 - tau2 + B22*dq2 + g2) - (p3*dMinv11_dq2 + p4*dMinv12_dq2)*(tau1 - B11*dq1 - B12*dq2 - g1 + dq2^2*h + 2*dq1*dq2*h) + (dq1^2*dh_dq2 + dg2_dq2)*(p3*Minv12 + p4*Minv22) - (p3*Minv11 + p4*Minv12)*(dq2^2*dh_dq2 + 2*dq1*dq2*dh_dq2 - dg1_dq2);
dp3 = (B11 - 2*dq2*h)*(p3*Minv11 + p4*Minv12) - p1 + (B21 + 2*dq1*h)*(p3*Minv12 + p4*Minv22);
dp4 = B22*(p3*Minv12 + p4*Minv22) - (p3*Minv11 + p4*Minv12)*(2*dq1*h - B12 + 2*dq2*h) - p2;
dp5 = - p3*Minv11 - p4*Minv12;
dp6 = - p3*Minv12 - p4*Minv22;

%% finally we define the state-costate dynamics

dX  = [dx1; dx2; dx3; dx4; dx5; dx6; dp1; dp2; dp3; dp4; dp5; dp6];



